# Copyright (C) 2006-2009 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from TagObject import TagObject
from Fence import Fence

TAG_NAME = "clusternode"

class ClusterNode(TagObject):
  def __init__(self):
    TagObject.__init__(self)
    self.TAG_NAME = TAG_NAME

  def getFenceNode(self):
    ret = None
    for child in self.getChildren():
        if child.getTagName() == 'fence':
            ret = child
            break
    if ret is None:
        ret = Fence()
        self.addChild(ret)
    return ret

  def getFenceLevels(self):
    # This method returns the set of 'method' objs. 'method' blocks represent
    # fence levels
    return self.getFenceNode().getChildren()

  def getMulticastNode(self):
    children = self.getChildren()
    for child in children:
      if child.getTagName() == "multicast":
        return child

    return None

  def getInterface(self):
    nd = self.getMulticastNode()
    if nd is None:
      return None
    else:
      return nd.getAttribute("interface")

  def setInterface(self, ifc):
    nd = self.getMulticastNode()
    if nd is None:
      return
    else:
      nd.addAttribute("interface", ifc)

  def isFenced(self):
    fence_sum = 0
    flevels = self.getFenceLevels()
    if len(flevels) == 0:
      return False

    for flevel in flevels:
      fence_sum = fence_sum + len(flevel.getChildren())

    if fence_sum > 0:
      return True
    else:
      return False

  def getVotes(self):
    try:
      return self.getAttribute('votes')
    except KeyError, e:
      return "1"
    except:
      return None

  def getID(self):
    try:
        return self.getAttribute('nodeid')
    except Exception, e:
        pass
    return "UNKNOWN"

  def removeFenceInstance(self, method, instance):
    """Remove fence instance from this node and given fence method."""
    for child in self.getChildren():
      if child.getTagName() == 'unfence':
        unfence = child
        for child_device in unfence.getChildren():
          # Try to find existing unfence device that mirrored fence device instance
          # to be removed and remove it, too.
          if len(set(instance.getAttributes().items()).difference(child_device.getAttributes().items())) == 0:
            unfence.removeChild(child_device)
            if len(unfence.getChildren()) == 0:
              # Remove the whole 'unfence' section from within 'node' section
              # if it is empty.
              self.removeChild(unfence)
            break
        break
    method.removeChild(instance)
