# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from pylons.i18n import ugettext as _

from luci.lib.ricci_communicator import RicciCommunicator
import luci.lib.ricci_queries as rq

from luci.lib.cluster_conf_helpers import get_cluster_conf_nodes
from luci.lib.db_helpers import get_cluster_task_status

def get_node_list(host, passwd):
    node_names = None
    cluster_name = None

    try:
        rc = RicciCommunicator(host, enforce_trust=False)   
        rc.trust()
        rc.auth(passwd)
        if not rc.authed():
            return { 'errors': [ _('Authentication to host "%s" failed') % host ]}

        cluster_name = rc.cluster_info()[0]
        if not cluster_name:
            return { 'errors': [_('Host "%s" is not a member of a cluster') % host ] }

        conf_xml = rq.getClusterConf(rc)
        if conf_xml:
            node_names = get_cluster_conf_nodes(conf_xml)

        if not node_names or len(node_names) < 1:
            return { 'errors': [ _('Unable to retrieve the list of cluster nodes from "%s"') % host ]}
    except Exception, e:
        return { 'errors': [ _('Unable to connect to host "%s": %s') % (host, e.message) ] }
    return { 'nodes': node_names, 'cluster': cluster_name }

def cluster_is_busy(cluster_name):
    return get_cluster_task_status(cluster_name)
