# Copyright (C) 2009-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

# -*- coding: utf-8 -*-

from tg import flash, redirect
from pylons.i18n import ugettext as _

from luci.model import DBSession
from luci.model.objects import Node, Cluster, Task

from luci.lib.ricci_helpers import send_batch_to_hosts, update_cluster_conf
from luci.lib.ricci_communicator import RicciCommunicator
from luci.lib.db_helpers import get_node_by_host
from luci.lib.ClusterConf.ClusterNode import ClusterNode
import luci.lib.ricci_queries as rq

import luci.lib.luci_tasks as luci_tasks

import transaction
import logging
log = logging.getLogger(__name__)

def validate_create_cluster_form(self, **kw):
    errors = []

    cluster_name = kw.get('cluster_name')
    if not cluster_name or len(cluster_name) < 1:
        flash(_('No cluster name was given'), 'error')
        return

    if len(cluster_name) > 15:
        flash(_('Cluster names must be less than 16 characters long'), 'error')
        return

    try:
        ret = int(DBSession.query(Cluster).filter_by(name=cluster_name).count())
        if ret != 0:
            flash(_('Luci is already managing a cluster named "%s"') % cluster_name, 'error')
            return
    except Exception, e:
        log.exception('Error querying the database while creating cluster %s' % cluster_name)
        flash(_('An error occurred while trying to query the luci database'), 'error')
        return

    enable_storage = kw.get('shared_storage') is not None
    reboot_nodes = kw.get('reboot_nodes') is not None
    download_pkgs = kw.get('download_pkgs') == 'download'

    num_nodes = kw.get('num_nodes')
    if num_nodes:
        try:
            num_nodes = int(num_nodes) + 1
        except ValueError, e:
            num_nodes = None

    if not num_nodes:
        flash(_('No number of nodes was given'), 'error')
        return

    nodes = []
    for cur in xrange(num_nodes):
        cur_host = kw.get('hostname%d' % cur)
        cur_passwd = kw.get('password%d' % cur)
        cur_port = kw.get('port%d' % cur)
        if not cur_host:
            continue
        if not cur_port:
            cur_port = '11111'
        if not cur_passwd:
            errors.append(_('No password was provided for node "%s"') % cur_host)
            continue

        nodes.append([ cur_host, cur_passwd, cur_port ])

    if len(nodes) < 1:
        errors.append(_('No cluster nodes were given'))

    if len(errors) > 0:
        flash(_('The following errors occurred while creating cluster "%s": %s')
            % (cluster_name, ', '.join(errors)), 'error')
        return

    cluster_db_obj = Cluster(name=cluster_name, display_name=cluster_name)

    node_list = []
    node_db_obj = {}
    task_db_obj = {}
    for node in nodes:
        try:
            rc = RicciCommunicator(node[0], port=int(node[2]), enforce_trust=False)
            rc.trust()
            rc.auth(node[1])
            if not rc.authed():
                errors.append('Authentication to the ricci agent at %s:%d failed'
                    % (node[0], node[2]))
                continue

            rc = RicciCommunicator(node[0])
            cur_cluster_name = rc.cluster_info()[0]
            if cur_cluster_name:
                errors.append('%s is already a member of a cluster named "%s"'
                    % (node[0], cur_cluster_name))
                continue

            node_list.append(node[0])
            node_db_obj[node[0]] = Node(node_name=node[0],
                                        display_name=node[0],
                                        hostname=node[0],
                                        ipaddr=node[0],
                                        port=node[2])
            DBSession.add(node_db_obj[node[0]])
        except Exception, e:
            log.exception('Error adding node %s while creating cluster %s'
                % (node[0], cluster_name))
            errors.append(str(e))

    if len(errors) > 0:
        flash(_('The following errors occurred while creating cluster "%s": %s')
            % (cluster_name, ', '.join(errors)), 'error')
        DBSession.rollback()
        return

    ret = send_batch_to_hosts(node_list, 10, rq.create_cluster,
                'rhel6', cluster_name, cluster_name,
                node_list, True, True, enable_storage, False,
                download_pkgs, None, reboot_nodes)

    for i in ret.iterkeys():
        if ret[i].has_key('error'):
           errors.append(_('Unable to connect to the ricci agent on node %s: %s') % (i, ret[i]['err_msg']))
        task_id = ret[i]['batch_result'][0]
        task_obj = Task(batch_id=task_id,
                        task_type=luci_tasks.TASK_CLUSTER_CREATE,
                        batch_status=-1,
                        redirect_url='/clusters/%s/' % cluster_name,
                        status_msg=_('Creating node "%s" for cluster "%s"') % (i, cluster_name))
        DBSession.add(task_obj)
        task_db_obj[i] = task_obj

    try:
        cluster_db_obj.nodes = node_db_obj.values()
        cluster_db_obj.tasks = task_db_obj.values()
        for i in node_db_obj.iterkeys():
            node_db_obj[i].tasks = [ task_db_obj[i] ]
        DBSession.add(cluster_db_obj)
    except Exception, e:
        log.exception('Error updating luci DB for cluster %s' % cluster_name)
        DBSession.rollback()
        flash(_('An error occurred during the creation of cluster "%s" while updating the luci database: %s') % (cluster_name, str(e)), 'error')
        return

    DBSession.flush()
    transaction.commit()
    flash(_('Creating the cluster "%s", this may take up to 30 seconds.  Please refresh screen to update status.') % cluster_name, 'info')
    redirect("/cluster")

# TODO: refactor to share common bits with the validation for
#       cluster create

def validate_node_add_form(model, db_obj, **kw):
    errors = []
    cluster_name = model.getClusterName()

    enable_storage = kw.get('shared_storage') is not None
    reboot_nodes = kw.get('reboot_nodes') is not None
    download_pkgs = kw.get('download_pkgs') == 'download'
    num_nodes = kw.get('num_nodes')

    if num_nodes:
        try:
            num_nodes = int(num_nodes) + 1
        except ValueError, e:
            num_nodes = None

    if not num_nodes:
        flash(_('No number of nodes was given'), 'error')
        return

    nodes = []
    for cur in xrange(num_nodes):
        cur_host = kw.get('hostname%d' % cur)
        cur_passwd = kw.get('password%d' % cur)
        cur_port = kw.get('port%d' % cur)
        if not cur_host:
            continue
        if not cur_port:
            cur_port = '11111'
        if not cur_passwd:
            errors.append(_('No password was provided for node "%s"') % cur_host)
            continue

        node_db_obj = get_node_by_host(cur_host)
        if node_db_obj is not None:
            errors.append(_('The host "%s" is already a member of cluster "%s"')
                % (cur_host, node_db_obj.cluster[0].name))

        nodes.append([ cur_host, cur_passwd, cur_port ])

    if len(nodes) < 1:
        errors.append(_('No cluster nodes were given'))

    last_node_id = 0
    try:
        last_node_id = max([int(n.getID()) for n in model.getNodes()])
    except:
        errors.append(_('Error getting node IDs for cluster %s') % cluster_name)
        log.exception('Error getting node IDs for cluster %s' % cluster_name)

    if len(errors) > 0:
        flash(_('The following errors occurred while adding nodes to cluster "%s": %s') % (cluster_name, ', '.join(errors)), 'error')
        return

    nodes_ptr = model.getClusterNodesPtr()
    node_list = []
    node_db_obj = {}
    task_db_obj = {}
    for node in nodes:
        try:
            rc = RicciCommunicator(node[0], port=int(node[2]), enforce_trust=False)
            rc.trust()
            rc.auth(node[1])
            if not rc.authed():
                errors.append('Authentication to the ricci agent at %s:%d failed'
                    % (node[0], node[2]))
                continue

            rc = RicciCommunicator(node[0], port=int(node[2]), enforce_trust=False)
            cur_cluster_name = rc.cluster_info()[0]
            if cur_cluster_name:
                errors.append('%s is already a member of a cluster named "%s"'
                    % (node[0], cur_cluster_name))
                continue

            node_list.append(node[0])
            node_db_obj[node[0]] = Node(node_name=node[0],
                                        display_name=node[0],
                                        hostname=node[0],
                                        ipaddr=node[0],
                                        port=node[2])

            last_node_id += 1
            new_node = ClusterNode()
            new_node.addAttribute('name', node[0])
            new_node.addAttribute('votes', '1')
            new_node.addAttribute('nodeid', str(last_node_id))
            nodes_ptr.addChild(new_node)
        except Exception, e:
            log.exception('Error adding node %s to cluster %s'
                % (node[0], cluster_name))
            errors.append(str(e))

    if len(errors) == 0:
        if update_cluster_conf(model) is not True:
            errors.append(_('Unable to update the cluster configuration on existing nodes'))

    if len(errors) > 0:
        flash(_('The following errors occurred while creating cluster "%s": %s')
            % (cluster_name, ', '.join(errors)), 'error')
        DBSession.rollback()
        return
    
    for i in node_db_obj.values():
        DBSession.add(i)
    db_obj.nodes.extend(node_db_obj.values())

    ret = send_batch_to_hosts(node_list, 10, rq.create_cluster_nodes,
                model, True, True, enable_storage, download_pkgs, reboot_nodes)

    for i in ret.iterkeys():
        if ret[i].has_key('error'):
           errors.append(_('Unable to connect to the ricci agent on node %s: %s') % (i, ret[i]['err_msg']))
        task_id = ret[i]['batch_result'][0]
        task_obj = Task(batch_id=task_id,
                        task_type=luci_tasks.TASK_CLUSTER_ADD_NODE,
                        batch_status=-1,
                        redirect_url='/clusters/%s/' % cluster_name,
                        status_msg=_('Creating node "%s" for cluster "%s"') % (i, cluster_name))
        DBSession.add(task_obj)
        task_db_obj[i] = task_obj

    try:
        db_obj.tasks.extend(task_db_obj.values())
        for i in node_db_obj.iterkeys():
            node_db_obj[i].tasks = [ task_db_obj[i] ]
    except Exception, e:
        log.exception('Error updating luci DB for cluster %s' % cluster_name)
        flash(_('An error occurred while adding nodes to cluster "%s": %s') % (cluster_name, str(e)), 'error')
        return

    DBSession.flush()
    transaction.commit()
    flash(_('Creating nodes for cluster "%s", this may take up to 30 seconds. Please refresh screen to update status.') % cluster_name, 'info')
